@@ -1,5 +1,4 @@
 import importlib.util
-import json
 import os
 import sys
 import time
@@ -7,27 +6,26 @@ from inspect import getsource
 from numbers import Number
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Callable, Dict, List, Optional, Union
 from zipfile import ZipFile
 
 from cognite.client import utils
 from cognite.client._api_client import APIClient
+from cognite.client.data_classes import TimestampRange
 
+from cognite.experimental._constants import HANDLER_FILE_NAME, LIST_LIMIT_CEILING, LIST_LIMIT_DEFAULT, MAX_RETRIES
 from cognite.experimental.data_classes import (
     Function,
     FunctionCall,
     FunctionCallList,
     FunctionCallLog,
+    FunctionFilter,
     FunctionList,
     FunctionSchedule,
+    FunctionSchedulesFilter,
     FunctionSchedulesList,
 )
 
-LIST_LIMIT_DEFAULT = 25
-LIST_LIMIT_CEILING = 10_000  # variable used to guarantee all items are returned when list(limit) is None, inf or -1.
-HANDLER_FILE_NAME = "handler.py"
-MAX_RETRIES = 5
-
 
 class FunctionsAPI(APIClient):
     _RESOURCE_PATH = "/functions"
@@ -50,6 +48,7 @@ class FunctionsAPI(APIClient):
         owner: Optional[str] = "",
         api_key: Optional[str] = None,
         secrets: Optional[Dict] = None,
+        env_vars: Optional[Dict] = None,
         cpu: Number = 0.25,
         memory: Number = 1.0,
     ) -> Function:
@@ -65,6 +64,8 @@ class FunctionsAPI(APIClient):
         - If the user gives one ore more secrets when creating the function, these are passed through the `secrets` argument. The API key can be access through `secrets["apikey"]`.\n
         - Data about the function call can be accessed via the argument `function_call_info`, which is a dictionary with keys `function_id` and, if the call is scheduled, `schedule_id` and `scheduled_time`.\n
 
+        The Python runtime version is 3.8.3.
+
         Args:
             name (str):                             The name of the function.
             folder (str, optional):                 Path to the folder where the function source code is located.
@@ -76,6 +77,7 @@ class FunctionsAPI(APIClient):
             owner (str, optional):                  Owner of this function. Typically used to know who created it.
             api_key (str, optional):                API key that can be used inside the function to access data in CDF.
             secrets (Dict[str, str]):               Additional secrets as key/value pairs. These can e.g. password to simulators or other data sources. Keys must be lowercase characters, numbers or dashes (-) and at most 15 characters. You can create at most 5 secrets, all keys must be unique, and cannot be apikey.
+            env_vars (Dict[str, str]):              Environment variables as key/value pairs. Keys can contain only letters, numbers or the underscore character. You can create at most 20 environment variables.
             cpu (Number):                           Number of CPU cores per function. Defaults to 0.25. Allowed values are in the range [0.1, 0.6].
             memory (Number):                        Memory per function measured in GB. Defaults to 1. Allowed values are in the range [0.1, 2.5].
 
@@ -133,6 +135,7 @@ class FunctionsAPI(APIClient):
             "functionPath": function_path,
             "cpu": float(cpu),
             "memory": float(memory),
+            "envVars": env_vars,
         }
         if external_id:
             function.update({"externalId": external_id})
@@ -164,11 +167,26 @@ class FunctionsAPI(APIClient):
         """
         self._delete_multiple(ids=id, external_ids=external_id, wrap_ids=True)
 
-    def list(self, limit: Optional[int] = LIST_LIMIT_DEFAULT) -> FunctionList:
-        """`List all functions. <https://docs.cognite.com/api/playground/#operation/get-function>`_
+    def list(
+        self,
+        name: str = None,
+        owner: str = None,
+        file_id: int = None,
+        status: str = None,
+        external_id_prefix: str = None,
+        created_time: Union[Dict[str, int], TimestampRange] = None,
+        limit: Optional[int] = LIST_LIMIT_DEFAULT,
+    ) -> FunctionList:
+        """`List all functions. <https://docs.cognite.com/api/playground/#operation/listFunctions>`_
 
         Args:
-            limit (int, optional): Maximum number of functions to list. Pass in -1, float('inf') or None to list all functions.
+            name (str): The name of the function.
+            owner (str): Owner of the function.
+            file_id (int): The file ID of the zip-file used to create the function.
+            status (str): Status of the function. Possible values: ["Queued", "Deploying", "Ready", "Failed"].
+            external_id_prefix (str): External ID prefix to filter on.
+            created_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
+            limit (int): Maximum number of functions to return. Pass in -1, float('inf') or None to list all.
 
         Returns:
             FunctionList: List of functions
@@ -181,14 +199,20 @@ class FunctionsAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> functions_list = c.functions.list()
         """
-        url = "/functions"
-
         if limit in [float("inf"), -1, None]:
             limit = LIST_LIMIT_CEILING
 
-        params = {"limit": limit}
-        res = self._get(url, params=params)
-        return FunctionList._load(res.json()["items"], cognite_client=self._cognite_client)
+        filter = FunctionFilter(
+            name=name,
+            owner=owner,
+            file_id=file_id,
+            status=status,
+            external_id_prefix=external_id_prefix,
+            created_time=created_time,
+        ).dump(camel_case=True)
+        res = self._post(url_path=f"{self._RESOURCE_PATH}/list", json={"filter": filter, "limit": limit})
+
+        return self._LIST_CLASS._load(res.json()["items"], cognite_client=self._cognite_client)
 
     def retrieve(self, id: Optional[int] = None, external_id: Optional[str] = None) -> Optional[Function]:
         """`Retrieve a single function by id. <https://docs.cognite.com/api/playground/#operation/post-api-playground-projects-project-context-functions-byids>`_
@@ -412,13 +436,13 @@ class FunctionCallsAPI(APIClient):
         """List all calls associated with a specific function id. Either function_id or function_external_id must be specified.
 
         Args:
-            limit (int, optional): Maximum number of function calls to list. Pass in -1, float('inf') or None to list all Function Calls.
             function_id (int, optional): ID of the function on which the calls were made.
             function_external_id (str, optional): External ID of the function on which the calls were made.
             status (str, optional): Status of the call. Possible values ["Running", "Failed", "Completed", "Timeout"].
             schedule_id (int, optional): Schedule id from which the call belongs (if any).
             start_time (Dict[str, int], optional): Start time of the call. Possible keys are `min` and `max`, with values given as time stamps in ms.
             end_time (Dict[str, int], optional): End time of the call. Possible keys are `min` and `max`, with values given as time stamps in ms.
+            limit (int, optional): Maximum number of function calls to list. Pass in -1, float('inf') or None to list all Function Calls.
 
         Returns:
             FunctionCallList: List of function calls
@@ -579,11 +603,22 @@ class FunctionSchedulesAPI(APIClient):
         utils._auxiliary.assert_exactly_one_of_id_or_external_id(id=id, external_id=None)
         return self._retrieve_multiple(ids=id, wrap_ids=True)
 
-    def list(self, limit: Optional[int] = LIST_LIMIT_DEFAULT) -> FunctionSchedulesList:
-        """`List all schedules associated with a specific project. <https://docs.cognite.com/api/playground/#operation/get-api-playground-projects-project-functions-schedules>`_
+    def list(
+        self,
+        name: str = None,
+        function_external_id: str = None,
+        created_time: Union[Dict[str, int], TimestampRange] = None,
+        cron_expression: str = None,
+        limit: Optional[int] = LIST_LIMIT_DEFAULT,
+    ) -> FunctionSchedulesList:
+        """`List all schedules associated with a specific project. <https://docs.cognite.com/api/playground/#operation/listFunctionSchedules>`_
 
         Args:
-            limit (int, optional): Maximum number of schedules to list. Pass in -1, float('inf') or None to list all schedules.
+            name (str): Name of the function schedule.
+            function_external_id (str): External ID of the function the schedules are linked to.
+            created_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
+            cron_expression (str): Cron expression.
+            limit (int): Maximum number of schedules to list. Pass in -1, float('inf') or None to list all.
 
         Returns:
             FunctionSchedulesList: List of function schedules
@@ -601,17 +636,21 @@ class FunctionSchedulesAPI(APIClient):
                 >>> from cognite.experimental import CogniteClient
                 >>> c = CogniteClient()
                 >>> func = c.functions.retrieve(id=1)
-                >>> schedules = func.list_schedules()
+                >>> schedules = func.list_schedules(limit=None)
 
         """
-        url = f"/functions/schedules"
-
         if limit in [float("inf"), -1, None]:
             limit = LIST_LIMIT_CEILING
 
-        params = {"limit": limit}
-        res = self._get(url, params=params)
-        return FunctionSchedulesList._load(res.json()["items"])
+        filter = FunctionSchedulesFilter(
+            name=name,
+            function_external_id=function_external_id,
+            created_time=created_time,
+            cron_expression=cron_expression,
+        ).dump(camel_case=True)
+        res = self._post(url_path=f"{self._RESOURCE_PATH}/list", json={"filter": filter, "limit": limit})
+
+        return self._LIST_CLASS._load(res.json()["items"], cognite_client=self._cognite_client)
 
     def create(
         self,
@@ -646,7 +685,7 @@ class FunctionSchedulesAPI(APIClient):
                     description="This schedule does magic stuff.")
 
         """
-        json = {
+        body = {
             "items": [
                 {
                     "name": name,
@@ -657,10 +696,10 @@ class FunctionSchedulesAPI(APIClient):
             ]
         }
         if data:
-            json["items"][0]["data"] = data
+            body["items"][0]["data"] = data
 
-        url = f"/functions/schedules"
-        res = self._post(url, json=json)
+        url = "/functions/schedules"
+        res = self._post(url, json=body)
         return FunctionSchedule._load(res.json()["items"][0])
 
     def delete(self, id: int) -> None:
@@ -681,9 +720,9 @@ class FunctionSchedulesAPI(APIClient):
                 >>> c.functions.schedules.delete(id = 123)
 
         """
-        json = {"items": [{"id": id,}]}
-        url = f"/functions/schedules/delete"
-        self._post(url, json=json)
+        body = {"items": [{"id": id}]}
+        url = "/functions/schedules/delete"
+        self._post(url, json=body)
 
     def get_input_data(self, id: int) -> Dict:
         """
