@@ -1,9 +1,13 @@
+import numbers
 from typing import Dict, List, Union
 
 from cognite.client.data_classes import ContextualizationJob
+from cognite.client.data_classes._base import CogniteResource
 
 from cognite.experimental._context_client import ContextAPI
 from cognite.experimental.data_classes import (
+    DiagramConvertResults,
+    DiagramDetectResults,
     PNIDConvertResults,
     PNIDDetectionList,
     PNIDDetectionPageList,
@@ -16,7 +20,7 @@ class PNIDParsingAPI(ContextAPI):
 
     def detect(
         self,
-        entities: List[Union[str, dict]],
+        entities: List[Union[str, dict, CogniteResource]],
         search_field: str = "name",
         name_mapping: Dict[str, str] = None,
         partial_match: bool = False,
@@ -25,6 +29,7 @@ class PNIDParsingAPI(ContextAPI):
         file_external_id: str = None,
     ) -> PNIDDetectResults:
         """Detect entities in a PNID.
+        The results are not written to CDF.
         Note: All users on this CDF subscription with assets read-all and files read-all capabilities in the project,
         are able to access the data sent to this endpoint.
 
@@ -37,18 +42,14 @@ class PNIDParsingAPI(ContextAPI):
             partial_match (bool): Allow for a partial match (e.g. missing prefix).
             min_tokens (int): Minimal number of tokens a match must be based on
         Returns:
-            PNIDDetectResults: Resulting queued job. Note that .results property of this job will block waiting for results."""
-
-        if not (
-            all([isinstance(entity, str) for entity in entities])
-            or all([isinstance(entity, dict) for entity in entities])
-        ):
-            raise ValueError("all the elements in entities must have same type (either str or dict)")
+            PNIDDetectResults: Resulting queued job. Note that .result property of this job will block waiting for results."""
 
         if file_id is None and file_external_id is None:
             raise ValueError("File id and file external id cannot both be none")
 
-        entities, entities_return = self._detect_before_hook(entities, search_field)
+        entities = [
+            entity.dump(camel_case=True) if isinstance(entity, CogniteResource) else entity for entity in entities
+        ]
 
         job = self._run_job(
             job_path="/detect",
@@ -57,45 +58,24 @@ class PNIDParsingAPI(ContextAPI):
             file_external_id=file_external_id,
             entities=entities,
             partial_match=partial_match,
+            search_field=search_field,
             name_mapping=name_mapping,
             min_tokens=min_tokens,
             job_cls=PNIDDetectResults,
         )
-        job.wait_for_completion()  # TODO: _detect_after_hook should be handled by PNIDDetectResults
-        if job.status == "Completed":
-            job = self._detect_after_hook(job, entities_return, search_field)
-        return job
-
-    @staticmethod
-    def _detect_before_hook(entities, search_field):
-        """To decide whether to use search_field or not and make sure the entities are of type List[str]"""
-        entities_return = None
-        if entities and isinstance(entities[0], dict):
-            entities_return = entities.copy()
-            entities = [entity.get(search_field) for entity in entities]
-        return entities, entities_return
-
-    @staticmethod
-    def _detect_after_hook(job, entities_return, search_field):
-        """Insert the entities into the result if search_field is used"""
-        if entities_return:
-            texts = {item.get("text") for item in job.result["items"]}
-            entities_return = [entity for entity in entities_return if entity.get(search_field) in texts]
-            for item in job.result["items"]:
-                item["entities"] = [entity for entity in entities_return if entity.get(search_field) == item["text"]]
         return job
 
     def extract_pattern(
         self, patterns: List[str], file_id: int = None, file_external_id: str = None
-    ) -> ContextualizationJob:
-        """Extract tags from P&ID based on pattern
+    ) -> PNIDDetectResults:
+        """Extract tags from P&ID based on pattern. The results are not written to CDF.
 
         Args:
             file_id (int): ID of the file, should already be uploaded in the same tenant.
             patterns (list): List of regular expression patterns to look for in the P&ID. See API docs for details.
 
         Returns:
-            ContextualizationJob: Resulting queued job. Note that .results property of this job will block waiting for results."""
+            PNIDDetectResults: Resulting queued job. Note that .result property of this job will block waiting for results."""
 
         if file_id is None and file_external_id is None:
             raise ValueError("File id and file external id cannot both be none")
@@ -106,20 +86,23 @@ class PNIDParsingAPI(ContextAPI):
             file_id=file_id,
             file_external_id=file_external_id,
             patterns=patterns,
+            job_cls=PNIDDetectResults,
         )
 
     def convert(
         self, items: List[Dict], grayscale: bool = None, file_id: int = None, file_external_id: str = None
     ) -> ContextualizationJob:
-        """Convert a P&ID to an interactive SVG where the provided annotations are highlighted
+        """Convert a P&ID to an interactive SVG where the provided annotations are highlighted.
+        The resulting SVG is not uploaded to CDF.
 
         Args:
             file_id (int): ID of the file, should already be uploaded in the same tenant.
             items (List[Dict]): List of entity annotations for entities detected in the P&ID.
+                For instance the resulting items from calling the detect or extract_pattern-method.
             grayscale (bool, optional): Return the SVG version in grayscale colors only (reduces the file size). Defaults to None.
 
         Returns:
-            ContextualizationJob: Resulting queued job. Note that .results property of this job will block waiting for results.
+            ContextualizationJob: Resulting queued job. Note that .result property of this job will block waiting for results.
         """
         if file_id is None and file_external_id is None:
             raise ValueError("File id and file external id cannot both be none")
@@ -148,3 +131,86 @@ class PNIDParsingAPI(ContextAPI):
             for item in res.json()["items"]
         ]
         return PNIDDetectionPageList(items, file_id=file_id)
+
+
+class DiagramsAPI(ContextAPI):
+    _RESOURCE_PATH = "/context/diagram"
+
+    @staticmethod
+    def _process_file_ids(ids: Union[List[int], int, None], external_ids: Union[List[str], str, None]) -> List:
+        if external_ids is None and ids is None:
+            raise ValueError("No ids specified")
+
+        if isinstance(ids, numbers.Integral):
+            ids = [ids]
+        elif isinstance(ids, list) or ids is None:
+            ids = ids or []
+        else:
+            raise TypeError("ids must be int or list of int")
+
+        if isinstance(external_ids, str):
+            external_ids = [external_ids]
+        elif isinstance(external_ids, list) or external_ids is None:
+            external_ids = external_ids or []
+        else:
+            raise TypeError("external_ids must be str or list of str")
+
+        ids = [{"fileId": id} for id in ids]
+        external_ids = [{"fileExternalId": external_id} for external_id in external_ids]
+        all_ids = ids + external_ids
+        return all_ids
+
+    def detect(
+        self,
+        entities: List[Union[dict, CogniteResource]],
+        search_field: str = "name",
+        partial_match: bool = False,
+        min_tokens: int = 2,
+        file_ids: List[int] = None,
+        file_external_ids: List[str] = None,
+    ) -> DiagramDetectResults:
+        """Detect entities in a PNID.
+        The results are not written to CDF.
+        Note: All users on this CDF subscription with assets read-all and files read-all capabilities in the project,
+        are able to access the data sent to this endpoint.
+
+        Args:
+            entities (List[Union[dict,CogniteResource]]): List of entities to detect
+            search_field (str): If entities is a list of dictionaries, this is the key to the values to detect in the PnId
+            partial_match (bool): Allow for a partial match (e.g. missing prefix).
+            min_tokens (int): Minimal number of tokens a match must be based on
+            file_ids (int): ID of the files, should already be uploaded in the same tenant.
+            file_external_ids: File external ids
+        Returns:
+            DiagramDetectResults: Resulting queued job. Note that .result property of this job will block waiting for results."""
+
+        entities = [
+            entity.dump(camel_case=True) if isinstance(entity, CogniteResource) else entity for entity in entities
+        ]
+
+        return self._run_job(
+            job_path="/detect",
+            status_path="/detect/",
+            items=self._process_file_ids(file_ids, file_external_ids),
+            entities=entities,
+            partial_match=partial_match,
+            search_field=search_field,
+            min_tokens=min_tokens,
+            job_cls=DiagramDetectResults,
+        )
+
+    def convert(self, detect_job: DiagramDetectResults) -> DiagramConvertResults:
+        """Convert a P&ID to interactive SVGs where the provided annotations are highlighted.
+
+        Args:
+            detect_job(DiagramConvertResults): detect job
+
+        Returns:
+            DiagramConvertResults: Resulting queued job. Note that .result property of this job will block waiting for results.
+        """
+        return self._run_job(
+            job_path="/convert",
+            status_path="/convert/",
+            items=detect_job.result["items"],
+            job_cls=DiagramConvertResults,
+        )
