@@ -2,7 +2,10 @@ import time
 from numbers import Number
 from typing import Dict, List, Optional, Union
 
-from cognite.client.data_classes._base import CogniteResource, CogniteResourceList
+from cognite.client.data_classes._base import CogniteFilter, CogniteResource, CogniteResourceList
+from cognite.client.data_classes.shared import TimestampRange
+
+from cognite.experimental._constants import LIST_LIMIT_DEFAULT
 
 
 class Function(CogniteResource):
@@ -20,6 +23,7 @@ class Function(CogniteResource):
         created_time (int): Created time in UNIX.
         api_key (str): Api key attached to the function.
         secrets (Dict[str, str]): Secrets attached to the function ((key, value) pairs).
+        env_vars (Dict[str, str]): User specified environment variables on the function ((key, value) pairs).
         cpu (Number): Number of CPU cores per function. Defaults to 0.25. Allowed values are in the range [0.1, 0.6].
         memory (Number): Memory per function measured in GB. Defaults to 1. Allowed values are in the range [0.1, 2.5].
         error(Dict[str, str]): Dictionary with keys "message" and "trace", which is populated if deployment fails.
@@ -39,6 +43,7 @@ class Function(CogniteResource):
         created_time: int = None,
         api_key: str = None,
         secrets: Dict = None,
+        env_vars: Dict = None,
         cpu: Number = None,
         memory: Number = None,
         error: Dict = None,
@@ -55,6 +60,7 @@ class Function(CogniteResource):
         self.created_time = created_time
         self.api_key = api_key
         self.secrets = secrets
+        self.env_vars = env_vars
         self.cpu = cpu
         self.memory = memory
         self.error = error
@@ -78,6 +84,7 @@ class Function(CogniteResource):
         schedule_id: Optional[int] = None,
         start_time: Optional[Dict[str, int]] = None,
         end_time: Optional[Dict[str, int]] = None,
+        limit: Optional[int] = LIST_LIMIT_DEFAULT,
     ) -> "FunctionCallList":
         """List all calls to this function.
 
@@ -86,23 +93,30 @@ class Function(CogniteResource):
             schedule_id (int, optional): Schedule id from which the call belongs (if any).
             start_time ([Dict[str, int], optional): Start time of the call. Possible keys are `min` and `max`, with values given as time stamps in ms.
             end_time (Dict[str, int], optional): End time of the call. Possible keys are `min` and `max`, with values given as time stamps in ms.
+            limit (int, optional): Maximum number of function calls to list. Pass in -1, float('inf') or None to list all Function Calls.
 
         Returns:
             FunctionCallList: List of function calls
         """
         return self._cognite_client.functions.calls.list(
-            function_id=self.id, status=status, schedule_id=schedule_id, start_time=start_time, end_time=end_time
+            function_id=self.id,
+            status=status,
+            schedule_id=schedule_id,
+            start_time=start_time,
+            end_time=end_time,
+            limit=limit,
         )
 
-    def list_schedules(self) -> "FunctionSchedulesList":
+    def list_schedules(self, limit: Optional[int] = LIST_LIMIT_DEFAULT) -> "FunctionSchedulesList":
         """`List all schedules associated with this function. <https://docs.cognite.com/api/playground/#operation/get-api-playground-projects-project-functions-schedules>`_
 
+        Args:
+            limit (int): Maximum number of schedules to list. Pass in -1, float('inf') or None to list all.
+
         Returns:
             FunctionSchedulesList: List of function schedules
         """
-        all_schedules = self._cognite_client.functions.schedules.list()
-        function_schedules = filter(lambda f: f.function_external_id == self.external_id, all_schedules)
-        return list(function_schedules)
+        return self._cognite_client.functions.schedules.list(function_external_id=self.external_id, limit=limit)
 
     def retrieve_call(self, id: int) -> "FunctionCall":
         """`Retrieve call by id. <https://docs.cognite.com/api/playground/#operation/get-api-playground-projects-project-functions-function_name-calls-call_id>`_
@@ -132,6 +146,24 @@ class Function(CogniteResource):
             setattr(self, attribute, latest_value)
 
 
+class FunctionFilter(CogniteFilter):
+    def __init__(
+        self,
+        name: str = None,
+        owner: str = None,
+        file_id: int = None,
+        status: str = None,
+        external_id_prefix: str = None,
+        created_time: Union[Dict[str, int], TimestampRange] = None,
+    ):
+        self.name = name
+        self.owner = owner
+        self.file_id = file_id
+        self.status = status
+        self.external_id_prefix = external_id_prefix
+        self.created_time = created_time
+
+
 class FunctionSchedule(CogniteResource):
     """A representation of a Cognite Function Schedule.
 
@@ -174,6 +206,20 @@ class FunctionSchedule(CogniteResource):
         return self._cognite_client.functions.schedules.get_input_data(id=self.id)
 
 
+class FunctionSchedulesFilter(CogniteFilter):
+    def __init__(
+        self,
+        name: str = None,
+        function_external_id: str = None,
+        created_time: Union[Dict[str, int], TimestampRange] = None,
+        cron_expression: str = None,
+    ):
+        self.name = name
+        self.function_external_id = function_external_id
+        self.created_time = created_time
+        self.cron_expression = cron_expression
+
+
 class FunctionSchedulesList(CogniteResourceList):
     _RESOURCE = FunctionSchedule
     _ASSERT_CLASSES = False
